#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Jun 24 13:01:59 2019

@author: pierrels
"""

import matplotlib.pyplot as plt
import numpy as np
from matplotlib import animation
import time

temps = time.time()

L = 1
n_L = 100

l = 0.3

T = 100
n_t = 100000

g = 3

dx = L / n_L
dt = T / n_t
n_l = min(int(l/dx), n_L)

n_L += 1

#Conditions initiales
u_init = np.sin(np.arange(0,L+dx,dx) * np.pi / 2)

theta_init = 0 * np.cos(np.arange(0, L+dx, dx))

v_init = np.zeros(n_L)

phi_init = np.zeros(n_L)

u = np.zeros((n_t, n_L))
theta = np.zeros((n_t, n_L))

u[0] = u_init
theta[0] = theta_init
u[1] = u_init + dt * v_init
theta[1] = theta_init + dt * phi_init

u[1,n_l] = max(u[1,n_l], 0)

M_u_u =  np.zeros((n_L, n_L))
M_u_theta = np.zeros((n_L, n_L))
M_theta_u = np.zeros((n_L, n_L))
M_theta_theta = np.zeros((n_L, n_L))

for k in range(1,n_L-1):
    M_u_u[k,k] = 2 - 2 * dt**2/dx**2
    M_u_u[k,k+1] = dt**2 / dx**2
    M_u_u[k,k-1] = dt**2 / dx**2
    
    M_u_theta[k,k+1] = - dt**2 / (2*dx)
    M_u_theta[k,k-1] = dt**2 / (2*dx)
    
    M_theta_theta[k,k] = 2 - g * 2 * dt**2 / dx**2 - dt**2
    M_theta_theta[k,k+1] = g * dt**2 / dx**2
    M_theta_theta[k,k-1] = g * dt**2 / dx**2
    
    M_theta_u[k,k+1] = dt**2 / (2*dx)
    M_theta_u[k,k-1] = -dt**2 / (2*dx)


M_u_u[0,0] = 2

M_theta_theta[0,0] = 2 - dt**2 / dx**2 - dt**2
M_theta_theta[0,1] = dt**2 / dx**2
M_theta_u[0,0] = - dt**2 / (dx)
M_theta_u[0,1] = dt**2 / (dx)

M_u_u[-1,-1] = 2 - 2*dt**2 / dx**2
M_u_u[-1,-2] = 2 * dt**2 / dx**2
M_u_theta[-1,-1] = 2 * dt**3 / dx**2 - dt**2 / (2*dx)
M_u_theta[-1,-2] = dt**2 / (2*dx)

M_theta_theta[-1,-1] = 2 - dt**2 / dx**2 + dt**3 / (2*dx) - dt**2
M_theta_theta[-1,-2] = dt**2 / dx**2


#f(x,t)

t_f = np.ones(n_L) * -0.000
t_f[0] = 0

def f(t,u,theta):
    return t_f

t_f_1 = np.zeros(n_L)
t_f_2 = np.zeros(n_L)
t_f_3 = np.zeros(n_L)
t_f_4 = np.zeros(n_L)
for k in range(0,n_L // 5, n_L // 5):
    t_f_1[k] = 1

def f_1(t,u,theta):
    pass

def timoshenko(n):
    theta[n] = M_theta_theta.dot(theta[n-1]) + M_theta_u.dot(u[n-1]) - theta[n-2]
    u[n] = M_u_u.dot(u[n-1]) + M_u_theta.dot(theta[n-1]) - u[n-2] + dt**2 * f(n*dt, u[n-1], theta[n-1]) + dt**2 * (M_theta_theta.dot(theta[n-1]) + M_theta_u.dot(u[n-1]) - theta[n-2])*u[n-1]
    u[n, n_l] = max(u[n, n_l],0)
#    u[n, n_l-1] = max(u[n, n_l-1],0)
#    u[n, n_l+1] = max(u[n, n_l+1],0)
#    u[n, n_l] = abs(u[n, n_l])

for n in range(2,n_t):
    timoshenko(n)
#    print("t = {}".format(n*dt))

print("done")

X = np.arange(0,L+dx,dx)

xmin = 0
xmax = L
ymin_u = np.min(u)
ymax_u = np.max(u)
ymin_theta = np.min(theta)
ymax_theta = np.max(theta)

fig, axes = plt.subplots(2,1)
ligne_u, = axes[0].plot([xmin,xmax],[ymin_u,ymax_u])
ligne_theta, = axes[1].plot([xmin,xmax],[ymin_theta,ymax_theta])
txt = axes[0].text(xmin, (u[0,0] + ymax_u)/2, "t = {}".format(int(0)/10))

axes[0].plot([n_l*dx, n_l*dx], [0, ymin_u])
axes[0].set_title("u")
axes[1].set_title("theta")

#def animate(i):
#    ligne_u.set_data(X, u[i,:])
#    ligne_theta.set_data(X,theta[i,:])
#    txt.set_text("t = {}".format(i * dt))
#
#ani = animation.FuncAnimation(fig, animate, frames=n_t, interval=1000/30)

fps = 30
ratio_vitesse = 1

def animate(i):
    ligne_u.set_data(X, u[int(i * n_t / (fps * T)),:])
    ligne_theta.set_data(X,theta[int(i * n_t / (fps * T)),:])
    txt.set_text("t = {}".format(int(int(i * n_t / (fps * T)) * dt * 10)/10))

ani = animation.FuncAnimation(fig, animate, frames=int(fps*T), interval=1000/(fps*ratio_vitesse))

Writer = animation.writers['ffmpeg']
writer = Writer(fps=30, metadata=dict(artist='Me'), bitrate=1800)
ani.save('im.mp4', writer=writer)

plt.show()